<?php
error_reporting(E_ALL | E_STRICT);
ini_set("track_errors", "On");

$INC = dirname(__FILE__) . "/../inc";
require_once(dirname(__FILE__) . "/config.php");
require_once($INC . "/init.inc.php");
require_once($INC . "/util.inc.php");
require_once($INC . "/keys.inc.php");
require_once($INC . "/sql.inc.php");
require_once($INC . "/struct.inc.php");
require_once($INC . "/fixes.inc.php");

rg_log_set_file("keys.log");

require_once("common.php");

$rg_sql_debug = 1;

// Defaults
$rg_admin_email = "rg@embedromix.ro";

$rg_ui = array("uid" => 1, "is_admin" => 0, "email" => "test@embedromix.ro");

$sql = "DELETE FROM keys";
$res = rg_sql_query($db, $sql);
rg_sql_free_result($res);

// insert a key 1
$key = "aaa 'bbb' first_key";
$key_id1 = rg_keys_add($db, $rg_ui, $key);
if ($key_id1 === FALSE) {
	echo "Cannot add key 1 (" . rg_keys_error() . ")!\n";
	exit(1);
}

// insert a key 2
$rg_ui['uid'] = 2;
$key = "aaa 'bbb' second_key";
$key_id2 = rg_keys_add($db, $rg_ui, $key);
if ($key_id2 === FALSE) {
	echo "Cannot add key 2 (" . rg_keys_error() . ")!\n";
	exit(1);
}


// test key file generation
$rg_scripts = "/a";
$rg_ssh_paras = "ssh1,ssh2,ssh3";
$r = rg_keys_regen($db);
if ($r === FALSE) {
	echo "Cannot regenerate keys (" . rg_keys_error() . ")!\n";
	exit(1);
}
$c = @file_get_contents("afile.txt");
if ($c === FALSE) {
	echo "Cannot regenerate file (afile.txt not found)!\n";
	exit(1);
}
$e = "command=\""
	. $rg_scripts . "/scripts/remote.sh 1 $key_id1\","
	. $rg_ssh_paras . " aaa 'bbb' first_key\n"
	. "command=\""
	. $rg_scripts . "/scripts/remote.sh 2 $key_id2\","
	. $rg_ssh_paras . " aaa 'bbb' second_key\n";
if (strcmp($c, $e) != 0) {
	echo "Generated file does not seems OK!\n";
	echo "e: $e\n";
	echo "c: $c\n";
	exit(1);
}

@unlink("afile.txt");


// delete a key
$rg_ui['uid'] = 1;
$list = array($key_id1 => "junk");
$r = rg_keys_remove($db, $rg_ui, $list);
if ($r === FALSE) {
	echo "Cannot remove key (" . rg_keys_error() . ")!\n";
	exit(1);
}


// test rg_max_ssh_keys
$rg_max_ssh_keys = 1;
// insert a key - should succeed
$rg_ui['uid'] = 10;
$key = "aaa 'bbb' first_key";
$key_id1 = rg_keys_add($db, $rg_ui, $key);
if ($key_id1 === FALSE) {
	echo "Cannot add key 1 (" . rg_keys_error() . ")!\n";
	exit(1);
}
// insert a key - must fail because overlimit
$key = "aaa 'bbb' second_key";
$key_id2 = rg_keys_add($db, $rg_ui, $key);
if ($key_id2 !== FALSE) {
	echo "Seems we can add more keys than allowed!\n";
	exit(1);
}



rg_sql_close($db);

echo "keys: OK\n";
?>
